@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import deprecation
 import mock
 
 from openstack.cluster.v1 import _proxy
@@ -107,6 +108,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
     def test_cluster_update(self):
         self.verify_update(self.proxy.update_cluster, cluster.Cluster)
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_add_nodes(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -118,6 +120,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_add_nodes_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.add_nodes",
@@ -125,6 +128,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      method_args=[mock_cluster, ["node1"]],
                      expected_args=[["node1"]])
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_del_nodes(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -136,13 +140,17 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_del_nodes_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.del_nodes",
                      self.proxy.cluster_del_nodes,
                      method_args=[mock_cluster, ["node1"]],
-                     expected_args=[["node1"]])
+                     method_kwargs={"key": "value"},
+                     expected_args=[["node1"]],
+                     expected_kwargs={"key": "value"})
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_replace_nodes(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -154,6 +162,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_replace_nodes_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.replace_nodes",
@@ -161,6 +170,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      method_args=[mock_cluster, {"node1": "node2"}],
                      expected_args=[{"node1": "node2"}])
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_scale_out(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -172,6 +182,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_scale_out_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.scale_out",
@@ -179,6 +190,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      method_args=[mock_cluster, 5],
                      expected_args=[5])
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_scale_in(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -190,6 +202,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_scale_in_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.scale_in",
@@ -217,6 +230,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      method_kwargs={'k1': 'v1', 'k2': 'v2'},
                      expected_kwargs={'k1': 'v1', 'k2': 'v2'})
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_attach_policy(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -230,6 +244,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_attach_policy_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.policy_attach",
@@ -239,6 +254,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      expected_args=["FAKE_POLICY"],
                      expected_kwargs={"k1": "v1", 'k2': "v2"})
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_detach_policy(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -250,6 +266,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_detach_policy_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.policy_detach",
@@ -257,6 +274,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      method_args=[mock_cluster, "FAKE_POLICY"],
                      expected_args=["FAKE_POLICY"])
 
+    @deprecation.fail_if_not_removed
     @mock.patch.object(proxy_base.BaseProxy, '_find')
     def test_cluster_update_policy(self, mock_find):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
@@ -270,6 +288,7 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
         mock_find.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER",
                                           ignore_missing=False)
 
+    @deprecation.fail_if_not_removed
     def test_cluster_update_policy_with_obj(self):
         mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
         self._verify("openstack.cluster.v1.cluster.Cluster.policy_update",
@@ -304,6 +323,17 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      method_args=["FAKE_CLUSTER"])
         mock_get.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER")
 
+    @deprecation.fail_if_not_removed
+    @mock.patch.object(proxy_base.BaseProxy, '_get_resource')
+    def test_cluster_operation(self, mock_get):
+        mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
+        mock_get.return_value = mock_cluster
+        self._verify("openstack.cluster.v1.cluster.Cluster.op",
+                     self.proxy.cluster_operation,
+                     method_args=["FAKE_CLUSTER", "dance"],
+                     expected_args=["dance"])
+        mock_get.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER")
+
     def test_node_create(self):
         self.verify_create(self.proxy.create_node, node.Node)
 
@@ -355,6 +385,17 @@ class TestClusterProxy(test_proxy_base2.TestProxyBase):
                      method_args=["FAKE_NODE"])
         mock_get.assert_called_once_with(node.Node, "FAKE_NODE")
 
+    @deprecation.fail_if_not_removed
+    @mock.patch.object(proxy_base.BaseProxy, '_get_resource')
+    def test_node_operation(self, mock_get):
+        mock_node = node.Node.new(id='FAKE_CLUSTER')
+        mock_get.return_value = mock_node
+        self._verify("openstack.cluster.v1.node.Node.op",
+                     self.proxy.node_operation,
+                     method_args=["FAKE_NODE", "dance"],
+                     expected_args=["dance"])
+        mock_get.assert_called_once_with(node.Node, "FAKE_NODE")
+
     def test_policy_create(self):
         self.verify_create(self.proxy.create_policy, policy.Policy)
 
